@@ -28,6 +28,7 @@ isc_boolean_t verbose_checks = ISC_FALSE; /* log each failure in CHECK() macro *
 
 /** Built-in defaults. */
 static const setting_t settings_default[] = {
+	{ "default_ttl",		default_uint(86400)		}, /* Seconds */
 	{ "uri",			no_default_string		}, /* User have to set this */
 	{ "connections",		default_uint(2)			},
 	{ "reconnect_interval",		default_uint(60)		},
@@ -61,6 +62,7 @@ static const setting_t settings_default[] = {
 	{ "serial_autoincrement",	default_string("")		},
 	{ "verbose_checks",		default_boolean(ISC_FALSE)	},
 	{ "directory",			default_string("")		},
+	{ "server_id",			default_string("")		},
 	end_of_settings
 };
 
@@ -354,7 +356,7 @@ cleanup:
  * @retval ISC_R_NOTFOUND Required setting was not found
  *                        in given set of settings.
  */
-static isc_result_t
+isc_result_t
 setting_unset(const char *const name, const settings_set_t *set)
 {
 	isc_result_t result;
@@ -532,7 +534,7 @@ settings_set_free(settings_set_t **set) {
 		mctx = (*set)->mctx;
 
 		if ((*set)->lock != NULL) {
-			isc_mutex_destroy((*set)->lock);
+			DESTROYLOCK((*set)->lock);
 			SAFE_MEM_PUT_PTR(mctx, (*set)->lock);
 		}
 
@@ -659,3 +661,23 @@ get_enum_description(const enum_txt_assoc_t *map, int value, const char **desc)
 	}
 	return ISC_R_NOTFOUND;
 }
+
+isc_result_t
+get_enum_value(const enum_txt_assoc_t *map, const char *description,
+	       int *value) {
+	const enum_txt_assoc_t *record;
+
+	REQUIRE(map != NULL);
+	REQUIRE(description != NULL);
+	REQUIRE(value != NULL);
+
+	for (record = map;
+	     record->description != NULL && record->value != -1;
+	     record++) {
+		if (strcasecmp(record->description, description) == 0) {
+			*value = record->value;
+			return ISC_R_SUCCESS;
+		}
+	}
+	return ISC_R_NOTFOUND;
+}
