@@ -39,6 +39,7 @@ struct task_element {
 static const isc_interval_t shutdown_timeout = { 3, 0 };
 
 /**
+ * @file syncrepl.c
  * @brief Synchronisation context.
  *
  * This structure provides information necessary for detecting the end
@@ -52,9 +53,13 @@ static const isc_interval_t shutdown_timeout = { 3, 0 };
  * zone. Each task involved in event processing is added to task list in
  * struct sync_ctx by sync_task_add() call.
  *
- * The initial synchronization is done when LDAP intermediate message
- * (with attribute refreshDone = TRUE) was received and all events generated
- * before this message were processed.
+ * The initial synchronization in LDAP_SYNC_REFRESH_ONLY mode is done
+ * when LDAP search result message was
+ * received and all events generated before this message were processed.
+ *
+ * The initial synchronization in LDAP_SYNC_REFRESH_AND_PERSIST mode is done
+ * when LDAP intermediate message (with attribute refreshDone = TRUE) was
+ * received and all events generated before this message were processed.
  *
  * LDAP intermediate message handler ldap_sync_intermediate() calls
  * sync_barrier_wait() and it sends sync_barrierev event to all involved tasks.
@@ -62,10 +67,17 @@ static const isc_interval_t shutdown_timeout = { 3, 0 };
  * events. As a result, all events generated before sync_barrier_wait() call
  * are processed before the call returns.
  *
- * @warning There are two assumptions:
+ * @warning There are three assumptions:
  * 	@li Each task processes events in FIFO order.
  * 	@li The task assigned to a LDAP instance or a DNS zone never changes.
+ * 	@li All code which depends on machine states is executed sequentially.
+ * 	    Asynchronous execution would lead to race conditions.
+ * 	    This currently works because all code depending on machine state
+ * 	    is directly or indirectly executed from ldap_sync_{init,poll}
+ * 	    functions and is synchronous.
  *
+ * @see ldap_sync_search_result()
+ * @see ldap_sync_intermediate()
  * @see ldap_sync_search_entry()
  */
 struct sync_ctx {
@@ -79,10 +91,11 @@ struct sync_ctx {
 	isc_condition_t			cond;	/**< for signal when task_cnt == 0 */
 	sync_state_t			state;
 	ldap_instance_t			*inst;
-	isc_event_t			*last_ev; /**< Last processed event */
 	ISC_LIST(task_element_t)	tasks;	/**< list of tasks processing
 						     events from initial
 						     synchronization phase */
+	isc_uint32_t			next_id;  /**< next sequential id */
+	isc_uint32_t			last_id;  /**< last processed event */
 };
 
 /**
@@ -111,6 +124,7 @@ finish(isc_task_t *task, isc_event_t *event) {
 	isc_result_t result = ISC_R_SUCCESS;
 	ldap_instance_t *inst = NULL;
 	sync_barrierev_t *bev = NULL;
+	sync_state_t new_state;
 
 	REQUIRE(ISCAPI_TASK_VALID(task));
 	REQUIRE(event != NULL);
@@ -119,10 +133,26 @@ finish(isc_task_t *task, isc_event_t *event) {
 	CHECK(manager_get_ldap_instance(bev->dbname, &inst));
 	log_debug(1, "sync_barrier_wait(): finish reached");
 	LOCK(&bev->sctx->mutex);
-	sync_state_change(bev->sctx, sync_finished, ISC_FALSE);
-	isc_condition_broadcast(&bev->sctx->cond);
+	switch (bev->sctx->state) {
+		case sync_configbarrier:
+			new_state = sync_datainit;
+			break;
+		case sync_databarrier:
+			new_state = sync_finished;
+			break;
+		case sync_configinit:
+		case sync_datainit:
+		case sync_finished:
+		default:
+			FATAL_ERROR(__FILE__, __LINE__,
+				    "sync_barrier_wait(): invalid state "
+				    "%u", bev->sctx->state);
+	}
+	sync_state_change(bev->sctx, new_state, ISC_FALSE);
+	BROADCAST(&bev->sctx->cond);
 	UNLOCK(&bev->sctx->mutex);
-	activate_zones(task, inst);
+	if (new_state == sync_finished)
+		activate_zones(task, inst);
 
 cleanup:
 	if (result != ISC_R_SUCCESS)
@@ -233,7 +263,7 @@ sync_barrierev_create(sync_ctx_t *sctx, const char *inst_name,
  * @param[in]	inst	LDAP instance associated with this synchronization ctx.
  * @param[out]	sctxp	The new synchronization context.
  *
- * @post state == sync_init
+ * @post state == sync_configinit
  * @post task_cnt == 1
  * @post tasks list contains the task
  */
@@ -264,7 +294,7 @@ sync_ctx_init(isc_mem_t *mctx, ldap_instance_t *inst, sync_ctx_t **sctxp) {
 
 	ISC_LIST_INIT(sctx->tasks);
 
-	sctx->state = sync_init;
+	sctx->state = sync_configinit;
 	CHECK(sync_task_add(sctx, ldap_instance_gettask(sctx->inst)));
 
 	CHECK(semaphore_init(&sctx->concurr_limit, LDAP_CONCURRENCY_LIMIT));
@@ -274,9 +304,10 @@ sync_ctx_init(isc_mem_t *mctx, ldap_instance_t *inst, sync_ctx_t **sctxp) {
 
 cleanup:
 	if (lock_ready == ISC_TRUE)
-		isc_mutex_destroy(&sctx->mutex);
+		DESTROYLOCK(&sctx->mutex);
 	if (cond_ready == ISC_TRUE)
-		isc_condition_init(&sctx->cond);
+		RUNTIME_CHECK(isc_condition_destroy(&sctx->cond)
+			      == ISC_R_SUCCESS);
 	if (refcount_ready == ISC_TRUE)
 		isc_refcount_destroy(&sctx->task_cnt);
 	MEM_PUT_AND_DETACH(sctx);
@@ -308,11 +339,11 @@ sync_ctx_free(sync_ctx_t **sctxp) {
 		isc_refcount_decrement(&sctx->task_cnt, NULL);
 		SAFE_MEM_PUT_PTR(sctx->mctx, taskel);
 	}
-	isc_condition_destroy(&sctx->cond);
+	RUNTIME_CHECK(isc_condition_destroy(&sctx->cond) == ISC_R_SUCCESS);
 	isc_refcount_destroy(&sctx->task_cnt);
 	UNLOCK(&sctx->mutex);
 
-	isc_mutex_destroy(&(*sctxp)->mutex);
+	DESTROYLOCK(&(*sctxp)->mutex);
 	MEM_PUT_AND_DETACH(*sctxp);
 }
 
@@ -343,14 +374,25 @@ sync_state_change(sync_ctx_t *sctx, sync_state_t new_state, isc_boolean_t lock)
 		LOCK(&sctx->mutex);
 
 	switch (sctx->state) {
-	case sync_init:
-		/* refresh phase is finished
-		 * and ldap_sync_intermediate() was called */
-		INSIST(new_state == sync_barrier);
+	case sync_configinit:
+		/* initial synchronization is finished
+		 * and ldap_sync_search_result() was called */
+		INSIST(new_state == sync_configbarrier);
 		break;
 
-	case sync_barrier:
-		/* sync_barrier_wait() finished */
+	case sync_configbarrier:
+		/* sync_barrier_wait(sync_configinit) finished */
+		INSIST(new_state == sync_datainit);
+		break;
+
+	case sync_datainit:
+		/* refresh phase is finished
+		 * and ldap_sync_intermediate() was called */
+		INSIST(new_state == sync_databarrier);
+		break;
+
+	case sync_databarrier:
+		/* sync_barrier_wait(sync_databarrier) finished */
 		INSIST(new_state == sync_finished);
 		break;
 
@@ -362,10 +404,44 @@ sync_state_change(sync_ctx_t *sctx, sync_state_t new_state, isc_boolean_t lock)
 	}
 
 	sctx->state = new_state;
+	log_debug(1, "sctx state %u reached", new_state);
 	if (lock == ISC_TRUE)
 		UNLOCK(&sctx->mutex);
 }
 
+/**
+ * Reset state of synchronization finite state machine.
+ * Reset can be done only before reaching state finished,
+ * i.e. when one of initial synchronizations in ldap_syncrepl_watcher failed.
+ *
+ * @warning The reset can reliably work only if all state transitions
+ *          are synchronous. This is necessary to prevent race conditions
+ *          between reset and events depending on particular state.
+ */
+void
+sync_state_reset(sync_ctx_t *sctx) {
+	REQUIRE(sctx != NULL);
+
+	LOCK(&sctx->mutex);
+
+	switch (sctx->state) {
+	case sync_configinit:
+	case sync_configbarrier:
+	case sync_datainit:
+	case sync_databarrier:
+		sctx->state = sync_configinit;
+		break;
+
+	case sync_finished:
+		/* state finished cannot be taken back, ever */
+	default:
+		fatal_error("invalid attempt to reset synchronization state");
+	}
+
+	log_debug(1, "sctx state %u reached (reset)", sctx->state);
+	UNLOCK(&sctx->mutex);
+}
+
 /**
  * @brief Add task to task list in synchronization context.
  *
@@ -388,7 +464,7 @@ sync_task_add(sync_ctx_t *sctx, isc_task_t *task) {
 	isc_task_attach(task, &newel->task);
 
 	LOCK(&sctx->mutex);
-	REQUIRE(sctx->state == sync_init);
+	REQUIRE(sctx->state == sync_configinit || sctx->state == sync_datainit);
 	ISC_LIST_APPEND(sctx->tasks, newel, link);
 	isc_refcount_increment0(&sctx->task_cnt, &cnt);
 	UNLOCK(&sctx->mutex);
@@ -407,7 +483,7 @@ cleanup:
  * @param[in,out]	sctx		Synchronization context
  * @param[in]		inst_name	LDAP instance name for given sctx
  *
- * @pre  sctx->state == sync_init
+ * @pre  sctx->state == sync_configinit || sync_datainit
  * @post sctx->state == sync_finished and all tasks processed all events
  *       enqueued before sync_barrier_wait() call.
  */
@@ -416,14 +492,34 @@ sync_barrier_wait(sync_ctx_t *sctx, const char *inst_name) {
 	isc_result_t result;
 	isc_event_t *ev = NULL;
 	sync_barrierev_t *bev = NULL;
+	sync_state_t barrier_state;
+	sync_state_t final_state;
 	task_element_t *taskel = NULL;
 	task_element_t *next_taskel = NULL;
 
 	LOCK(&sctx->mutex);
-	REQUIRE(sctx->state == sync_init);
+	REQUIRE(sctx->state == sync_configinit || sctx->state == sync_datainit);
 	REQUIRE(!EMPTY(sctx->tasks));
 
-	sync_state_change(sctx, sync_barrier, ISC_FALSE);
+	switch (sctx->state) {
+		case sync_configinit:
+			barrier_state = sync_configbarrier;
+			final_state = sync_datainit;
+			break;
+		case sync_datainit:
+			barrier_state = sync_databarrier;
+			final_state = sync_finished;
+			break;
+		case sync_configbarrier:
+		case sync_databarrier:
+		case sync_finished:
+		default:
+			FATAL_ERROR(__FILE__, __LINE__,
+				    "sync_barrier_wait(): invalid state "
+				    "%u", sctx->state);
+	}
+
+	sync_state_change(sctx, barrier_state, ISC_FALSE);
 	for (taskel = next_taskel = HEAD(sctx->tasks);
 	     taskel != NULL;
 	     taskel = next_taskel) {
@@ -437,8 +533,8 @@ sync_barrier_wait(sync_ctx_t *sctx, const char *inst_name) {
 	}
 
 	log_debug(1, "sync_barrier_wait(): wait until all events are processed");
-	while (sctx->state != sync_finished)
-		isc_condition_wait(&sctx->cond, &sctx->mutex);
+	while (sctx->state != final_state)
+		WAIT(&sctx->cond, &sctx->mutex);
 	log_debug(1, "sync_barrier_wait(): all events were processed");
 
 cleanup:
@@ -492,20 +588,29 @@ sync_concurr_limit_signal(sync_ctx_t *sctx) {
 }
 
 /**
- * Wait until given event ev is processed.
+ * Send ISC event to specified task and optionally wait until given event
+ * is processed.
  *
- * End of event processing has to be signalled by
- * sync_event_signal() call.
+ * End of event processing has to be signaled by
+ * @see sync_event_signal() call.
  */
 isc_result_t
-sync_event_wait(sync_ctx_t *sctx, isc_event_t *ev) {
+sync_event_send(sync_ctx_t *sctx, isc_task_t *task, ldap_syncreplevent_t **ev,
+		isc_boolean_t synchronous) {
 	isc_result_t result;
 	isc_time_t abs_timeout;
+	isc_uint32_t seqid;
+	isc_boolean_t locked = ISC_FALSE;
 
 	REQUIRE(sctx != NULL);
 
 	LOCK(&sctx->mutex);
-	while (sctx->last_ev != ev) {
+	locked = ISC_TRUE;
+	/* overflow is not a problem as long as the modulo is smaller than
+	 * constant used by sync_concurr_limit_wait() */
+	(*ev)->seqid = seqid = ++sctx->next_id % 0xffffffff;
+	isc_task_send(task, (isc_event_t **)ev);
+	while (synchronous == ISC_TRUE && sctx->last_id != seqid) {
 		if (ldap_instance_isexiting(sctx->inst) == ISC_TRUE)
 			CLEANUP_WITH(ISC_R_SHUTTINGDOWN);
 
@@ -518,7 +623,8 @@ sync_event_wait(sync_ctx_t *sctx, isc_event_t *ev) {
 	result = ISC_R_SUCCESS;
 
 cleanup:
-	UNLOCK(&sctx->mutex);
+	if (locked == ISC_TRUE)
+		UNLOCK(&sctx->mutex);
 	return result;
 }
 
@@ -526,12 +632,12 @@ cleanup:
  * Signal that given syncrepl event was processed.
  */
 void
-sync_event_signal(sync_ctx_t *sctx, isc_event_t *ev) {
+sync_event_signal(sync_ctx_t *sctx, ldap_syncreplevent_t *ev) {
 	REQUIRE(sctx != NULL);
 	REQUIRE(ev != NULL);
 
 	LOCK(&sctx->mutex);
-	sctx->last_ev = ev;
+	sctx->last_id = ev->seqid;
 	BROADCAST(&sctx->cond);
 	UNLOCK(&sctx->mutex);
 }
