@@ -17,13 +17,17 @@ typedef enum sync_state		sync_state_t;
 typedef struct sync_barrierev	sync_barrierev_t;
 
 enum sync_state {
-	sync_init,	/**< initial synchronisation in progress;
-			     expecting LDAP intermediate message
-			     with refreshDone = TRUE */
-	sync_barrier,	/**< waiting until all tasks process events generated
-			     during initial synchronisation phase*/
-	sync_finished	/**< initial synchronisation done; all events generated
-			     during initial synchronisation were processed */
+	sync_configinit,	/**< initial config synchronization in progress;
+				     expecting LDAP result message */
+	sync_configbarrier,	/**< waiting until all tasks process events
+				     generated during initial synchronization */
+	sync_datainit,		/**< initial data synchronization in progress;
+				     expecting LDAP intermediate message
+				     with refreshDone = TRUE */
+	sync_databarrier,	/**< waiting until all tasks process data events
+				     generated during initial synchronization */
+	sync_finished	/**< initial synchronization done; all events generated
+			     during initial synchronization were processed */
 };
 
 isc_result_t
@@ -38,6 +42,9 @@ sync_state_get(sync_ctx_t *sctx, sync_state_t *statep) ATTR_NONNULLS;
 void
 sync_state_change(sync_ctx_t *sctx, sync_state_t new_state, isc_boolean_t lock) ATTR_NONNULLS;
 
+void
+sync_state_reset(sync_ctx_t *sctx) ATTR_NONNULLS;
+
 isc_result_t
 sync_task_add(sync_ctx_t *sctx, isc_task_t *task) ATTR_NONNULLS ATTR_CHECKRESULT;
 
@@ -51,9 +58,10 @@ void
 sync_concurr_limit_signal(sync_ctx_t *sctx) ATTR_NONNULLS;
 
 isc_result_t
-sync_event_wait(sync_ctx_t *sctx, isc_event_t *ev) ATTR_NONNULLS ATTR_CHECKRESULT;
+sync_event_send(sync_ctx_t *sctx, isc_task_t *task, ldap_syncreplevent_t **ev,
+		isc_boolean_t synchronous) ATTR_NONNULLS ATTR_CHECKRESULT;
 
 void
-sync_event_signal(sync_ctx_t *sctx, isc_event_t *ev) ATTR_NONNULLS;
+sync_event_signal(sync_ctx_t *sctx, ldap_syncreplevent_t *ev) ATTR_NONNULLS;
 
 #endif /* SYNCREPL_H_ */
